@@ -229,11 +229,11 @@ _matmul(gf * a, gf * b, gf * c, unsigned n, unsigned k, unsigned m) {
  * Return non-zero if singular.
  */
 static void
-_invert_mat(gf* src, unsigned k) {
-    gf c, *p;
-    unsigned irow = 0;
-    unsigned icol = 0;
-    unsigned row, col, i, ix;
+_invert_mat(gf* src, size_t k) {
+    gf c;
+    size_t irow = 0;
+    size_t icol = 0;
+    size_t row, col, i, ix;
 
     unsigned* indxc = (unsigned*) malloc (k * sizeof(unsigned));
     unsigned* indxr = (unsigned*) malloc (k * sizeof(unsigned));
@@ -306,7 +306,8 @@ _invert_mat(gf* src, unsigned k) {
          */
         id_row[icol] = 1;
         if (memcmp (pivot_row, id_row, k * sizeof (gf)) != 0) {
-            for (p = src, ix = 0; ix < k; ix++, p += k) {
+            gf *p = src;
+            for (ix = 0; ix < k; ix++, p += k) {
                 if (ix != icol) {
                     c = p[icol];
                     p[icol] = 0;
@@ -320,6 +321,10 @@ _invert_mat(gf* src, unsigned k) {
         if (indxr[col-1] != indxc[col-1])
             for (row = 0; row < k; row++)
                 SWAP (src[row * k + indxr[col-1]], src[row * k + indxc[col-1]], gf);
+    free(indxc);
+    free(indxr);
+    free(ipiv);
+    free(id_row);
 }
 
 /*
@@ -418,6 +423,11 @@ fec_new(unsigned short k, unsigned short n) {
 
     fec_t *retval;
 
+    assert(k >= 1);
+    assert(n >= 1);
+    assert(n <= 256);
+    assert(k <= n);
+
     if (fec_initialized == 0)
         init_fec ();
 
